package ro.thehunters.digi.recipeManager.flags;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.Validate;
import org.bukkit.inventory.meta.ItemMeta;

import ro.thehunters.digi.recipeManager.Tools;

public class FlagItemLore extends Flag
{
    // Flag definition and documentation
    
    private static final FlagType TYPE;
    protected static final String[] A;
    protected static final String[] D;
    protected static final String[] E;
    
    static
    {
        TYPE = FlagType.ITEMLORE;
        
        A = new String[]
        {
            "{flag} <text>",
        };
        
        D = new String[]
        {
            "Adds a line to result's lore (description)",
            "",
            "Supports colors (e.g. <red>, <blue>, &4, &F, etc).",
            "",
            "You can also use these variables:",
            "  {player}         = crafter's name or '(nobody)' if not available",
            "  {playerdisplay}  = crafter's display name or '(nobody)' if not available",
            "  {result}         = the result item name or '(nothing)' if recipe failed.",
            "  {recipename}     = recipe's custom or autogenerated name or '(unknown)' if not available",
            "  {recipetype}     = recipe type or '(unknown)' if not available",
            "  {inventorytype}  = inventory type or '(unknown)' if not available",
            "  {world}          = world name of event location or '(unknown)' if not available",
            "  {x}              = event location's X coord or '(?)' if not available",
            "  {y}              = event location's Y coord or '(?)' if not available",
            "  {z}              = event location's Z coord or '(?)' if not available",
        };
        
        E = new String[]
        {
            "{flag} <red>Awesome item",
            "{flag} <magic>some scrambled text on line 2",
            "{flag} <gray>Crafted at {world}:{x},{y},{z}",
        };
    }
    
    // Flag code
    
    private List<String> lore = new ArrayList<String>();
    
    public FlagItemLore()
    {
    }
    
    public FlagItemLore(FlagItemLore flag)
    {
        lore.addAll(flag.lore);
    }
    
    @Override
    public FlagItemLore clone()
    {
        return new FlagItemLore(this);
    }
    
    @Override
    public FlagType getType()
    {
        return TYPE;
    }
    
    public List<String> getLore()
    {
        return lore;
    }
    
    public void setLore(List<String> lore)
    {
        Validate.notNull(lore, "The 'lore' argument must not be null!");
        
        this.lore.clear();
        
        for(String value : lore)
        {
            addLore(value);
        }
    }
    
    public void addLore(String value)
    {
        lore.add(Tools.parseColors(value, false));
    }
    
    @Override
    protected boolean onParse(String value)
    {
        addLore(value);
        
        return true;
    }
    
    @Override
    protected void onPrepare(Args a)
    {
        if(!a.hasResult())
        {
            a.addCustomReason("Need result!");
            return;
        }
        
        ItemMeta meta = a.result().getItemMeta();
        List<String> lore = meta.getLore();
        
        if(lore == null)
        {
            lore = new ArrayList<String>();
        }
        
        for(String line : this.lore)
        {
            if(line != null && !line.isEmpty())
            {
                lore.add(a.parseVariables(line));
            }
        }
        
        meta.setLore(lore);
        
        a.result().setItemMeta(meta);
    }
}
