package ro.thehunters.digi.recipeManager.flags;

import org.bukkit.inventory.meta.ItemMeta;

import ro.thehunters.digi.recipeManager.Tools;

public class FlagItemName extends Flag
{
    // Flag definition and documentation
    
    private static final FlagType TYPE;
    protected static final String[] A;
    protected static final String[] D;
    protected static final String[] E;
    
    static
    {
        TYPE = FlagType.ITEMNAME;
        
        A = new String[]
        {
            "{flag} <text or false>",
        };
        
        D = new String[]
        {
            "Changes result's display name.",
            "",
            "Supports colors (e.g. <red>, <blue>, &4, &F, etc).",
            "",
            "You can also use these variables:",
            "  {player}         = crafter's name or '(nobody)' if not available",
            "  {playerdisplay}  = crafter's display name or '(nobody)' if not available",
            "  {result}         = the result item name or '(nothing)' if recipe failed.",
            "  {recipename}     = recipe's custom or autogenerated name or '(unknown)' if not available",
            "  {recipetype}     = recipe type or '(unknown)' if not available",
            "  {inventorytype}  = inventory type or '(unknown)' if not available",
            "  {world}          = world name of event location or '(unknown)' if not available",
            "  {x}              = event location's X coord or '(?)' if not available",
            "  {y}              = event location's Y coord or '(?)' if not available",
            "  {z}              = event location's Z coord or '(?)' if not available",
        };
        
        E = new String[]
        {
            "{flag} <light_pruple>Weird Item",
            "{flag} <yellow>{player}'s Sword",
        };
    }
    
    // Flag code
    
    private String name;
    
    public FlagItemName()
    {
    }
    
    public FlagItemName(FlagItemName flag)
    {
        name = flag.name;
    }
    
    @Override
    public FlagItemName clone()
    {
        return new FlagItemName(this);
    }
    
    @Override
    public FlagType getType()
    {
        return TYPE;
    }
    
    public String getName()
    {
        return name;
    }
    
    public void setName(String name)
    {
        this.name = name;
    }
    
    @Override
    protected boolean onParse(String value)
    {
        setName(value);
        return true;
    }
    
    @Override
    protected void onPrepare(Args a)
    {
        if(!a.hasResult())
        {
            a.addCustomReason("Needs result!");
            return;
        }
        
        ItemMeta meta = a.result().getItemMeta();
        
        meta.setDisplayName(getName() == null ? null : Tools.parseColors(a.parseVariables(getName()), false));
        
        a.result().setItemMeta(meta);
    }
}
